"""
Created by adam on 2/23/20
"""
from unittest import TestCase

from CanvasHacks.DAOs.sqlite_dao import SqliteDAO
from tests.TestingBase import TestingBase
from tests.factories.ModelFactories import student_factory
from tests.factories.PeerReviewedFactories import activity_data_factory
from faker import Faker
fake = Faker()
import datetime
import pytz
from CanvasHacks.Models.status_record import ComplexStatusRecord
from CanvasHacks.TimeTools import current_utc_timestamp


__author__ = 'adam'

if __name__ == '__main__':
    pass


class TestStatusRecord( TestingBase ):

    def setUp(self):
        self.config_for_test()
        self.dao = SqliteDAO()
        print("Connected to testing db")
        self.session = self.dao.session

        self.activity_data = activity_data_factory()
        self.target_student = student_factory()
        self.reviewing_student = student_factory()
        self.reviewed_student = student_factory()
        self.activity_id = fake.random.randint(1111, 99999)

    def test_initial_record_creation( self ):
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.session.add( rec )
        self.session.commit()

        self.assertIsInstance( rec, ComplexStatusRecord )

        # Check
        r = self.session.query( ComplexStatusRecord ) \
            .filter( ComplexStatusRecord.content_assignment_id == self.activity_id ) \
            .all()
        self.assertEqual(len(r), 1, "One record returned")
        self.assertEqual(r[0].student_id, self.target_student.id)
        self.assertEqual(r[0].content_assignment_id, self.activity_id)

    def test_is_under_review( self ):
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.session.add( rec )
        self.session.commit()

        self.assertIsInstance( rec, ComplexStatusRecord )
        self.assertFalse(rec.is_under_review())

        rec.reviewed_by = self.reviewing_student.id
        self.assertTrue(rec.is_under_review())

    def test_add_reviewee( self ):
        # prep
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.assertTrue(rec.reviewer_of is None)
        self.assertTrue(rec.reviewer_assigned_on is None)

        # call
        rec.add_reviewee(self.reviewed_student)

        # check
        self.assertIsInstance(rec.reviewer_assigned_on, datetime.datetime)
        self.assertEqual(rec.reviewer_of, self.reviewed_student.id)

    def test_add_reviewer( self ):
        # prep
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.assertTrue(rec.reviewed_by is None)

        # call
        rec.add_reviewer(self.reviewing_student)

        # check
        self.assertEqual(rec.reviewed_by, self.reviewing_student.id)

    def test_record_content_assignment_submission_no_date_provided( self ):
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.assertTrue(rec.content_assignment_submitted is None)

        # call
        rec.record_content_assignment_submission()

        # check
        self.assertIsInstance(rec.content_assignment_submitted, datetime.datetime)
        self.assertEqual(rec.content_assignment_submitted.tzinfo, pytz.utc, "Timestamp is utc")

    def test_record_content_assignment_submission_date_provided( self ):
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.assertTrue(rec.content_assignment_submitted is None)

        submission_ts = current_utc_timestamp()

        # call
        rec.record_content_assignment_submission(submission_ts)

        # check
        self.assertIsInstance(rec.content_assignment_submitted, datetime.datetime)
        self.assertEqual(rec.content_assignment_submitted.tzinfo, pytz.utc, "Timestamp is utc")
        self.assertEqual(rec.content_assignment_submitted, submission_ts, "Passed-in timestamp set")

    def test_record_sending_metareview_results( self ):
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.assertTrue(rec.metareview_results_on is None)

        # call
        rec.record_sending_metareview_results()

        # check
        self.assertIsInstance(rec.metareview_results_on, datetime.datetime)
        self.assertEqual(rec.metareview_results_on.tzinfo, pytz.utc, "Timestamp is utc")

    def test_record_sending_review_results( self ):
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.assertTrue(rec.review_results_on is None)

        # call
        rec.record_sending_review_results()

        # check
        self.assertIsInstance(rec.review_results_on, datetime.datetime)
        self.assertEqual(rec.review_results_on.tzinfo, pytz.utc, "Timestamp is utc")

    def test_record_wait_notification( self ):
        rec = ComplexStatusRecord( student_id=self.target_student.id, content_assignment_id=self.activity_id )
        self.assertTrue(rec.wait_notification_on is None)

        # call
        rec.record_wait_notification()

        # check
        self.assertIsInstance(rec.wait_notification_on, datetime.datetime)
        self.assertEqual(rec.wait_notification_on.tzinfo, pytz.utc, "Timestamp is utc")




"""
Creates all the assignments in a unit
Created by adam on 3/1/20
"""
__author__ = 'adam'

import pandas as pd

import CanvasHacks.environment as env
from CanvasHacks.Models.model import StoreMixin
from CanvasHacks.Definitions.unit import Unit

LOC = '{}/Box Sync/TEACHING/Phil 305 Business ethics/Phil305 S20'.format(
    env.ROOT )  # placeholder for where the access codes are stored

DETAILS_FP = "{}/Unit details.xlsx".format( LOC )


class UnitDefinitionsLoader( StoreMixin ):

    def __init__( self, **kwargs ):
        self.handle_kwargs( **kwargs )

    def load( self, filepath ):
        with pd.ExcelFile( filepath ) as xls:
            self.unit_details = pd.read_excel( xls, 'details' )
            self.start_dates = pd.read_excel( xls, 'startdates' )
            # start_dates.start = start_dates.apply(lambda x: pd.Timestamp(x.start, tz='US/Pacific'), axis=1)
            self.start_dates.start = self.start_dates.apply( lambda x: pd.Timestamp( x.start ), axis=1 )

            # Clean up the text
            self.unit_details.activity_title = self.unit_details.activity_title.str.strip()

    def run( self, course ):
        for u in self.units:
            for a in u.components:
                maker = CreationHandlerFactory.make( a, course )
                # print( maker.creation_dict )
                maker.create()

    def make_unit( self, unit_number, start ):
        u = Unit( [ ], unit_number )
        for activity_type in u.component_types:
            # Holds the data we'll use to create the activity
            v = {}
            # Represents a date which gets incremented for each step
            d = start

            # Get the dates sorted out
            # Have to do this first, since the penalizer for some activities
            # will require the due date when it initializes
            deets = self.unit_details[ self.unit_details.activity_title == activity_type.title_base ]

            v[ 'unlock_at' ] = d + pd.Timedelta( '1 minutes' )
             # = unlock_at

            d += pd.Timedelta( '{} days'.format( deets.due_days_from_start.values[ 0 ] ) )
            v[ 'due_at' ] = d - pd.Timedelta( '1 minute' )
            # setattr( activity, 'due_at', due_at )
            # v[ 'due_at' ] = due_at

            d += pd.Timedelta( '{} days'.format( deets.lock_after_due.values[ 0 ] ) )
            v[ 'lock_at' ] = d - pd.Timedelta( '1 minute' )


            # Instantiate
            activity = activity_type( unit_number=u.unit_number, due_at=v[ 'due_at' ] )
            #         print(activity.title_base)
            # deets = self.unit_details[ self.unit_details.activity_title == activity.title_base ]
            #         print(deets)

            setattr( activity, 'unit_number', unit_number )

            v['title'] = activity.make_title

            # Set time values

            setattr( activity, 'unlock_at', v[ 'unlock_at' ] )
            setattr( activity, 'lock_at', v['lock_at'] )

            # unlock_at = d + pd.Timedelta( '1 minutes' )
            # v[ 'unlock_at' ] = unlock_at


            # d += pd.Timedelta( '{} days'.format( deets.lock_after_due.values[ 0 ] ) )
            # lock_at = d - pd.Timedelta( '1 minute' )
            # setattr( activity, 'lock_at', lock_at )
            # v[ 'lock_at' ] = lock_at

            # set point values
            points_possible = deets.points.values[ 0 ]
            setattr( activity, 'points_possible', points_possible )
            v[ 'points_possible' ] = points_possible

            # set text
            # description = activity.description
            # setattr( activity, 'description', description )
            v[ 'description' ] = activity.description

            activity.creation_dict = v

            u.components.append( activity )

        return u

    def create_units( self ):
        self.units = [ ]
        for i, row in self.start_dates.iterrows():
            print( row.unit )
            u = self.make_unit( row.unit, row.start )
            #         u.components.append( activity_type( **v) )
            self.units.append( u )

        return self.units


class CreationHandlerFactory:
    @classmethod
    def make( cls, activity, course, **kwargs ):
        if activity.creation_type == 'assignment':
            return AssignmentCreator( activity, course )
            # course.create_assignment( a.creation_dict )
        if activity.creation_type == 'quiz':
            return QuizCreator( activity, course )
            # course.create_quiz( a.creation_dict )
        if activity.creation_type == 'discussion':
            return DiscussionCreator( activity, course )
            # course.create_discussion_topic( a.creation_dict )
        if activity.creation_type == 'survey':
            return SurveyCreator( activity, course )


class IActivityCreator:
    """Parent of everything which creates stuff on canvas
    """

    def create( self ):
        raise NotImplementedError


class AssignmentCreator( IActivityCreator ):

    def __init__( self, activity, course, **kwargs ):
        self.course = course
        self.activity = activity

    @property
    def creation_dict( self ):
        d = self.activity.creation_dict
        # add fields specific to this knd of activity

        return d

    def create( self ):
        return self.course.create_assignment( self.creation_dict )


class QuizCreator( IActivityCreator ):

    def __init__( self, activity, course, **kwargs ):
        self.course = course
        self.activity = activity

    @property
    def creation_dict( self ):
        d = self.activity.creation_dict
        # add fields specific to this knd of activity
        return d

    def create( self ):
        return self.course.create_quiz( self.creation_dict )


class DiscussionCreator( IActivityCreator ):

    def __init__( self, activity, course, **kwargs ):
        self.course = course
        self.activity = activity

    @property
    def creation_dict( self ):
        d = self.activity.creation_dict
        # add fields specific to this knd of activity

        return d

    def create( self ):
        return self.course.create_assignment( self.creation_dict )


class SurveyCreator( IActivityCreator ):

    def __init__( self, activity, course, **kwargs ):
        self.course = course
        self.activity = activity

    @property
    def creation_dict( self ):
        d = self.activity.creation_dict
        # add fields specific to this knd of activity
        return d

    def create( self ):
        return self.course.create_assignment( self.creation_dict )


if __name__ == '__main__':
    pass



"""
Created by adam on 12/24/19
"""
__author__ = 'adam'


class Model(object):
    """Parent class of all model objects"""

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            try:
                setattr(self, key, value)
            except AttributeError as e:
                print("Problem setting {} to {} \n".format(key, value), e)



    #
    # for k in kwargs.keys():
    #         # print(k, kwargs[k])
    #         setattr(self, k,  kwargs[k])


if __name__ == '__main__':
    pass


class StoreMixin( object ):

    def handle_kwargs( self, **kwargs ):
        for key, value in kwargs.items():
            try:
                setattr(self, key, value)
            except AttributeError as e:
                print("Problem setting {} to {} \n".format(key, value), e)



"""
todo This is very much tied to my own filesystem and needs. Should be abstracted.
Created by adam on 10/26/18
"""
__author__ = 'adam'

import csv
import json
import os

from CanvasHacks import environment

DATA_FOLDER = '%s/data' % environment.CONFIG.proj_base
IGNORE_FILE = "%s/ignore.csv" % DATA_FOLDER
JOURNAL_ARCHIVE_FOLDER = "%s/Journals" % environment.CONFIG.archive_folder
LOG_FOLDER = environment.CONFIG.log_folder


# ----------------- create files

def journal_folder_name( journal_name, course_id, journal_folder=JOURNAL_ARCHIVE_FOLDER ):
    """Creates a folder to store student work for the unit"""
    folder_name = journal_name.replace( '(', '' ).replace( ')', '' ).replace( ' ', '-' )
    return "%s/%s-%s" % (journal_folder, course_id, folder_name)


def get_journal_folders( journal_archive=JOURNAL_ARCHIVE_FOLDER ):
    """Constructs paths to all journal folders and returns the list
        example result: [
            '/Users/adam/Box Sync/Phil 305 Business ethics/Student work/41181-Journal-week-2',
             '/Users/adam/Box Sync/Phil 305 Business ethics/Student work/41181-Journal-week-3',
             ....
    """
    journal_folders = [ ]
    for root, dirs, files in os.walk(journal_archive):
        for d in dirs:
            journal_folders.append( os.path.join( root, d ) )
    return journal_folders


def make_folder_list( sections, week_num ):
    return [ "%s-Journal-week-%s" % (s, week_num) for s in sections ]


# counts from each week
def calculate_journal_counts( journal_folders, data_file_name='all-submissions' ):
    """Returns the count of journals for each week"""
    cnt = [ ]
    for r in journal_folders:
        with open( "%s/%s.json" % (r, data_file_name), 'r' ) as f:
            j = json.load( f )
        cnt.append( (r.split( '/' )[ -1: ], len( j )) )
    return cnt


def load_words_to_ignore( file=IGNORE_FILE ):
    words = [ ]
    with open( file, 'r' ) as csvfile:
        reader = csv.DictReader( csvfile )  # , delimiter=',', quotechar='|')
        for row in reader:
            #             print (row['journals'])
            words.append( row[ 'journals' ] )
    return words


def week_key_gen(start=2, stop=16):
    """Create an iterator for all the keys to the various dictionaries"""
    for i in range(start, stop):
        yield "w%s" % i


if __name__ == '__main__':
    pass



"""
Created by adam on 12/26/19
"""
from unittest.mock import MagicMock, create_autospec

from faker import Faker

from tests.TestingBase import TestingBase

fake = Faker()

from CanvasHacks.Messaging.skaa import *
from CanvasHacks.Definitions.skaa import *
from tests.factories.PeerReviewedFactories import activity_data_factory, unit_factory
from tests.factories.ModelFactories import student_factory
from CanvasHacks.Repositories.students import StudentRepository
from tests.factories.RepositoryMocks import ContentRepositoryMock

__author__ = 'adam'


def prompt_response_factory( number=3 ):
    return [ { 'prompt': fake.sentence(), 'response': fake.paragraph() } for _ in range( 0, number ) ]

#
# class TestMake_conversation_data( TestCase ):
#     def test_make_conversation_data( self ):
#         self.fail()


class TestStudentNotices( TestingBase ):

    def test_make_prompt_and_response( self ):
        w = prompt_response_factory()
        j = make_prompt_and_response( w )

        self.assertTrue( len( j ) > 1, "Created string is non empty" )


    # def test_make_notice( self ):
    #     w = prompt_response_factory()
    #     d = { 'name': fake.name(),
    #           'response_list': w,
    #           'review_assignment_name': fake.job(),
    #           'access_code': fake.slug()
    #           }
    #     j = make_notice(d)
    #     # self.assertEquals(  j, 1, "Created string is non empty" )
    #     self.assertTrue( len( j ) > 1, "Created string is non empty" )


# def make_expected_message_data(activity_inviting_to_complete, template, receiving_student, content, other=None):
#     d = {
#         'intro': activity_inviting_to_complete.email_intro,
#
#         'name': get_first_name( receiving_student ),
#
#         # Formatted work for sending
#         'responses': content,
#
#         # Add any materials from me
#         'other': other,
#
#         # Add code and link to do reviewing unit
#         'review_assignment_name': activity_inviting_to_complete.name,
#         'access_code': activity_inviting_to_complete.access_code,
#         'review_url': activity_inviting_to_complete.html_url,
#         'due_date': activity_inviting_to_complete.string_due_date
#     }
#
#     message = template.format( **d )


class TestSkaaMessenger(TestingBase):

    def setUp(self):
        self.config_for_test()
        self.unit = unit_factory()
        self.activity_data = activity_data_factory()
        self.activity = Activity(**self.activity_data)
        # student recieiving the message
        self.receiving_student = student_factory()
        self.reviewed_student_work = fake.text()

        self.studentRepo = StudentRepository()
        self.studentRepo.get_student = MagicMock( return_value=self.receiving_student )
        self.contentRepo = ContentRepositoryMock()
        self.statusRepo = create_autospec( StatusRepository )

    def test__make_message_data(self):
        self.contentRepo.get_formatted_work_by = MagicMock(return_value=self.reviewed_student_work)
        review_assign = MagicMock( assessor_id=self.receiving_student.id )

        # parent class doesn't define message_template
        testing_template = """{intro} {name} {responses} {other} 
        {review_assignment_name} {access_code_message} {review_url} {due_date}
        """

        self.obj = SkaaMessenger(self.activity, self.studentRepo, self.contentRepo, self.statusRepo)

        self.obj.message_template = testing_template
        self.obj.activity_inviting_to_complete = self.activity

        # Call
        result = self.obj._make_message_data( self.receiving_student, self.reviewed_student_work )

        # Check
        self.assertIsInstance(self.obj.student_repository, StudentRepository)
        expected_keys = ['student_id', 'subject', 'body']
        for k in result.keys():
            self.assertIn(k, expected_keys, "Result contains expected keys")

    # @patch('CanvasHacks.PeerReviewed.Notifications.ConversationMessageSender')
    # def test_notify( self, senderMock ):
    #     self.contentRepo.get_formatted_work_by = MagicMock(return_value=self.work)
    #     review_assign = MagicMock( assessor_id=self.receiving_student.id )
    #
    #     # parent class doesn't define message_template
    #     testing_template = """{intro} {name} {responses} {other}
    #         {review_assignment_name} {access_code} {review_url} {due_date}
    #         """
    #
    #     self.obj = SkaaMessenger(self.activity_inviting_to_complete, self.studentRepo, self.contentRepo)
    #
    #     self.obj.message_template = testing_template
    #
    #     # Call
    #     result = self.obj.notify([review_assign], send=True )
    #
    #     # Check
    #     # Only going to check most general things here. Specific stuff
    #     # like content and who gets what will be tested in
    #     # tests for child classes
    #     self.assertTrue(self.contentRepo.get_formatted_work_by.called, "Content repo method called")
    #     self.assertTrue(self.studentRepo.get_student_record.called, "Student repo method called")
    #     # Returned expected stuff
    #     self.assertTrue( len( result ) == 1, "Something was returned" )
    #     # Sender method called
    #     senderMock.send.assert_called()



"""
Created by adam on 1/18/20
"""
from unittest import TestCase
from tests.TestingBase import TestingBase
from CanvasHacks.DAOs.sqlite_dao import SqliteDAO
from tests.factories.PeerReviewedFactories import unit_factory
from CanvasHacks.Errors.review_pairings import NoAvailablePartner, AllAssigned

from tests.factories.ModelFactories import student_factory
from CanvasHacks.Repositories.reviewer_associations import AssociationRepository
from faker import Faker
fake = Faker()

__author__ = 'adam'

if __name__ == '__main__':
    pass


class TestAssociationRepository( TestingBase ):
    def setUp(self):
        dao = SqliteDAO()
        self.session = dao.session
        # self.student_ids = [i for i in range(0,5)]
        self.create_new_and_preexisting_students()

        self.unit = unit_factory()
        self.activity = self.unit.initial_work
        self.activity_id = self.activity.id
        self.obj = AssociationRepository(dao, self.activity)

    def test__make_associations_single_submissions( self ):
        # Students only submit once
        # id case
        assoc = self.obj._make_associations(self.student_ids)
        self.assertEqual(len(self.students), len(assoc) , "single submission; ids")

        # obj case
        assoc = self.obj._make_associations(self.students)
        self.assertEqual(len(self.students), len(assoc) , "single submission; objects" )

    def test__make_associations_3_submissions( self ):
        # Students only submit once
        # id case
        self.student_ids = [i for i in range(0,3)]
        assoc = self.obj._make_associations(self.student_ids)
        self.assertEqual(3, len(assoc) , "single submission; ids")

        # obj case
        self.students = [student_factory() for i in range(0,3)]
        assoc = self.obj._make_associations(self.students)
        self.assertEqual(3, len(assoc) , "single submission; objects" )

    def test__make_associations_double_submissions( self ):
        # Students only submit twice adjacent
        s = []
        for so in self.students:
            s.append(so)
            s.append(so)
        # obj case
        assoc = self.obj._make_associations(s)
        for a, b in assoc:
            self.assertNotEqual(a, b,  "self-unit multi submission; objects"  )

    def test__make_associations_double_submissions_ids( self ):
        s = []
        for so in self.student_ids:
            s.append(so)
            s.append(so)
        # obj case
        assoc = self.obj._make_associations(s)
        for a, b in assoc:
            self.assertNotEqual(a, b,  "self-unit multi submission; objects"  )

    def test_assign_reviewer_raises_when_all_submitters_already_assigned( self ):
        preexisting_pairings = self.create_preexisting_review_pairings(self.activity_id, self.preexisting_students)

        with self.assertRaises(AllAssigned):
            self.obj.assign_reviewers(self.preexisting_students)

    def test_assign_reviewer_raises_when_only_one_submitter( self ):
        with self.assertRaises(NoAvailablePartner):
            self.obj.assign_reviewers(self.preexisting_students[:1])


#
    # def test_get_associations( self ):
    #     self.fail()
    #
    # def test_create( self ):
    #     self.fail()
    #
    # def test_create_association( self ):
    #     self.fail()
    #
    # def test_get_reviewer( self ):
    #     self.fail()
    #
    # def test_get_submitter( self ):
    #     self.fail()
    #
    # def test_get_for_assignment( self ):
    #     self.fail()



"""
Created by adam on 2/18/20
"""
from CanvasHacks.Models.model import StoreMixin

__author__ = 'adam'


class IGrader(StoreMixin):
    """Parent which defines common methods for any
    class in charge of grading student work

    todo Calling classes should be keeping a record of details of how grade assigned
    """
    def __init__(self, **kwargs):
        self.graded = [ ]
        self.handle_kwargs(**kwargs)

    def grade( self ):
        """Carry out grading on internally stored data
        from the repos it holds"""
        raise NotImplementedError

    # def log( self ):
    #     """
    #     :return:
    #     """
    #     raise NotImplementedError

    @property
    def activity( self ):
        return self.work_repo.activity

#
# class WorkingGrader(IGrader):
#
#     def grade( self ):
#         # for student
#         total_score = 0
#
#         # compute the initial total score of the activity
#         # this is the job of the GradingMethod
#
#         down = 0
#         # get a negative float representing the pct the total score
#         # should be adjusted down
#         for penalizer in self.penalizers:
#             down += penalizer.get_penalty()
#
#         up = 0
#         for correction in self.corrections:
#             # get a positive float representing the pct the total score
#             # should be adjusted up
#             up += correction.analyze()
#
#         adj = up + down
#
#         #todo careful about adj = 0
#
#         if self.using_fudge_points:
#             fudge_points = total_score * adj
#
#         if self.using_percentage:
#             pct = 100 * adj
#
#         # using total points
#         score = total_score * adj
#

class AllQuestionsEmpty(Exception):
    """
    Raised when a student's submission has all values empty.
    This can be caught and a check performed to see whether the
    activity depends on another student doing their part
    """
    pass


if __name__ == '__main__':
    pass


"""
Created by adam on 12/26/19
"""
from CanvasHacks import environment as env
from CanvasHacks.Errors.messaging import MessageDataCreationError
from CanvasHacks.Messaging.SendTools import send_message_to_student
from CanvasHacks.Messaging.base import SkaaMessenger
from CanvasHacks.Messaging.templates import METAREVIEW_CONTENT_TEMPLATE, \
    METAREVIEW_NOTICE_TEMPLATE, REVIEW_NOTICE_TEMPLATE
from CanvasHacks.Models.student import get_first_name
from CanvasHacks.Definitions.unit import Unit
from CanvasHacks.Repositories.status import StatusRepository, IStatusRepository
from CanvasHacks.TimeTools import getDateForMakingFileName

__author__ = 'adam'

if __name__ == '__main__':
    pass


class PeerReviewInvitationMessenger( SkaaMessenger ):
    """Handles sending message containg student work from the initial content unit to the person who will conduct the peer review
    """
    message_template = REVIEW_NOTICE_TEMPLATE

    def __init__( self, unit: Unit, student_repository, content_repository,
                  status_repositories: IStatusRepository ):

        self.activity_inviting_to_complete = unit.review

        super().__init__( unit, student_repository, content_repository, status_repositories )
        # self.message_template = REVIEW_NOTICE_TEMPLATE

    def prepare_message( self, review_assignment, other=None ):
        """This looks up the appropriate data for a review
        unit and returns what will be the message body
        """
        try:
            # We are going to send the original work to the assessor
            # who will do the peer review
            receiving_student = self.student_repository.get_student( review_assignment.assessor_id )

            # The assessee did the work that we want to send
            # to the assessor
            content = self.content_repository.get_formatted_work_by( review_assignment.assessee_id )

            return self._make_message_data( receiving_student, content, other=None )

        except Exception as e:
            # todo exception handling
            print( e )
            raise MessageDataCreationError( review_assignment )


class MetareviewInvitationMessenger( SkaaMessenger ):
    """Handles sending message containing feedback from the peer reviewer
    to the author of the content assignment with invite to do metareview
    """
    message_template = METAREVIEW_NOTICE_TEMPLATE

    def __init__( self, unit: Unit, student_repository, content_repository,
                  status_repositories: StatusRepository ):

        # The activity we are notifying about
        self.activity_inviting_to_complete = unit.metareview

        super().__init__( unit, student_repository, content_repository, status_repositories )

    def prepare_message( self, review_assignment, other=None ):
        """This looks up the appropriate data for a review
        unit and returns what will be the message body
        """
        try:
            if review_assignment is None:
                raise MessageDataCreationError(review_assignment)

            # We are going to send the peer review feedback
            # created by the assessor to the student who was
            # assessed in the peer review stage
            receiving_student = self.student_repository.get_student( review_assignment.assessee_id )

            # The assessor did the work that we want to send
            # to the assessee
            content = self.content_repository.get_formatted_work_by( review_assignment.assessor_id )

            return self._make_message_data( receiving_student, content, other=None )

        except Exception as e:
            # todo exception handling
            print( e )
            raise MessageDataCreationError( review_assignment )


class FeedbackFromMetareviewMessenger( SkaaMessenger ):
    """Sends the contents of the metareview to the student who r
    did the initial peer review
    """
    message_template = METAREVIEW_CONTENT_TEMPLATE
    subject = "Unit {}: Feedback on your peer review"
    intro = "Here is the feedback the author gave on your peer review. "

    def __init__( self, unit: Unit, student_repository, content_repository,
                  status_repositories: IStatusRepository ):

        # None because sending feedback
        self.activity_inviting_to_complete = None

        super().__init__( unit, student_repository, content_repository, status_repositories )

        self.email_subject = self.subject.format(self.unit.unit_number)


    def prepare_message( self, review_assignment, other=None ):
        """This looks up the appropriate data for a review
        unit and returns what will be the message body
        """
        try:
            # We are going to send the metareview feedback
            # created by the assessee (original author) to the student who did
            # the assessing in the peer review stage
            receiving_student = self.student_repository.get_student( review_assignment.assessor_id )

            # The assessee (original author) did the work that we want to send
            # to the assessor (peer reviewer)
            content = self.content_repository.get_formatted_work_by( review_assignment.assessee_id )

            d = {
                'intro': self.intro,

                'name': get_first_name( receiving_student ),

                # Formatted work for sending
                'responses': content,

                # Add any materials from me
                'other': "",
            }

            body = self.message_template.format( **d )

            # can't use the usual self.make_message_data because won't have all the expected fields
            return {
                'student_id': receiving_student.id,
                'subject': self.email_subject,
                'body': body
            }

        except Exception as e:
            # todo exception handling
            print( e )
            raise MessageDataCreationError( review_assignment )


# --------------------------------------- OLD

def make_prompt_and_response( response_list ):
    temp = """
            ------
    Prompt: {prompt}

    Their response:
    {response}
            ------
    """
    rs = [ temp.format( **r ) for r in response_list ]
    return " ".join( rs )


def make_notice( data ):
    """Should define
    name, responses, review_assignment_name, access_code
    """
    return REVIEW_NOTICE_TEMPLATE.format( **data )


# old
def make_metareview_notice( data ):
    return METAREVIEW_NOTICE_TEMPLATE.format( **data )


# old
def metareview_send_message_to_reviewers( review_assignments, studentRepo, contentRepo, activity, send=False ):
    # # Load list of ReviewAssociation objects representing who reviews whom
    # review_assigns = associationRepo.get_associations(activity_inviting_to_complete)
    # print("loaded {} student reviewer assignments".format(len(review_assigns)))
    log_file = "{}/{}-metareview-message-log.txt".format( env.LOG_FOLDER, getDateForMakingFileName() )
    with open( log_file, 'a' ) as f:
        for rev in review_assignments:
            try:
                assessee = studentRepo.get_student_record( rev.assessee_id )

                content = contentRepo.get_formatted_work_by( rev.assessor_id )

                d = {
                    'intro': activity.email_intro,

                    'name': get_first_name( assessee ),

                    # Formatted work for sending
                    'responses': content,

                    # Add any materials from me
                    'other': '',

                    # Add code and link to do reviewing unit
                    'review_assignment_name': activity.name,
                    'access_code': activity.access_code,
                    'review_url': activity.html_url,
                    'due_date': activity.string_due_date
                }

                message = make_notice( d )

                f.write( "\n=========\n {}".format( message ) )

                if send:
                    subject = activity.email_subject
                    m = send_message_to_student(
                        student_id=rev.assessee_id,
                        subject=subject,
                        body=message )
                    print( m )
                else:
                    print( message )
            except Exception as e:
                # todo Replace with raise LookupError and hook handler
                f.write( "\n=========\n {}".format( e ) )
                print( e )


def review_send_message_to_reviewers( review_assignments, studentRepo, contentRepo, activity, send=False ):
    # THIS IS UNUSABLE. MUST FIX ERROR

    # # Load list of ReviewAssociation objects representing who reviews whom
    # review_assigns = associationRepo.get_associations(activity_inviting_to_complete)
    # print("loaded {} student reviewer assignments".format(len(review_assigns)))

    for rev in review_assignments:
        try:
            assessor = studentRepo.get_student_record( rev.assessor_id )

            content = contentRepo.get_formatted_work( rev.assessee_id )

            d = {
                'intro': activity.email_intro,

                'name': get_first_name( assessor ),

                # Formatted work for sending
                'responses': content,

                # Add any materials from me
                'other': '',

                # Add code and link to do reviewing unit
                'review_assignment_name': activity.name,
                'access_code': activity.access_code,
                'review_url': activity.html_url,
                'due_date': activity.string_due_date
            }

            message = make_notice( d )

            if send:
                subject = activity.email_subject
                # fix this you fucking idiot
                m = send_message_to_student( student_id=rev.assessor_id, subject=subject, body=message )
                print( m )
            else:
                print( message )
        except Exception as e:
            # todo Replace with raise LookupError and hook handler
            print( e )
